<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin;

use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps;
use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps\Edit;
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\OnlineMaps as OnlineMapsRepository;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\SettingsMessageId;
use Locr\WordPress\Plugin\LocrMaps\TypedSequence;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the ONLINEmaps list table.
 */
final class OnlineMapsTable extends \WP_List_Table {
	/**
	 * This function displays the checkbox column.
	 *
	 * @param OnlineMap $item The item.
	 */
	public function column_cb( $item ): string {
		return sprintf(
			'<input type="checkbox" name="id[]" value="%s" />',
			$item->id->value
		);
	}

	/**
	 * This function displays the table.
	 *
	 * @param OnlineMap $item The item.
	 * @param string    $column_name The column name.
	 * @return mixed
	 */
	public function column_default( $item, $column_name ): mixed {
		return match ( $column_name ) {
			'id' => $item->id->value,
			'name' => (string) $item->name,
			'shortcode' => '[locrmaps id="' . $item->id->value . '"]',
			'style' => (string) $item->style,
			default => null
		};
	}

	/**
	 * This function displays the ID column.
	 *
	 * @param OnlineMap $item The item.
	 */
	protected function column_id( $item ): string {
		$admin_page_url = admin_url( 'admin.php' );

		$id = $item->id->value;

		$page = filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'page', FILTER_DEFAULT );

		$actions                  = array();
		$query_args_edit          = array(
			'page'     => $page,
			'action'   => 'edit',
			'id'       => $id,
			'_wpnonce' => wp_create_nonce( Edit::NONCE_ACTION ),
		);
		$edit_onlinemaps_item_url = esc_url( add_query_arg( $query_args_edit, $admin_page_url ) );
		$actions['edit']          = sprintf( '<a href="%s">%s</a>', $edit_onlinemaps_item_url, __( 'Edit', 'default' ) );

		$query_args_delete = array(
			'page'     => $page,
			'action'   => 'delete',
			'id'       => $id,
			'_wpnonce' => wp_create_nonce( OnlineMaps::NONCE_ACTION_DELETE ),
		);
		$delete_url        = esc_url( add_query_arg( $query_args_delete, $admin_page_url ) );
		$actions['trash']  = sprintf( '<a href="%s">%s</a>', $delete_url, __( 'Delete', 'default' ) );

		return sprintf( '%1$s %2$s', $id, $this->row_actions( $actions ) );
	}

	/**
	 * This function displays the icon URL column.
	 *
	 * @return array<mixed>
	 */
	public function get_bulk_actions(): array {
		return array(
			'bulk-delete' => __( 'Delete', 'default' ),
		);
	}

	/**
	 * This function initializes the icon list table.
	 *
	 * @return array<mixed> The columns.
	 */
	public function get_columns(): array {
		return array(
			'cb'        => '<input type="checkbox" />',
			'id'        => __( 'ID', 'locrmaps-textdomain' ),
			'name'      => __( 'Name', 'locrmaps-textdomain' ),
			'style'     => __( 'Style', 'locrmaps-textdomain' ),
			'shortcode' => __( 'Shortcode', 'locrmaps-textdomain' ),
		);
	}

	/**
	 * This function gets the data.
	 *
	 * @return TypedSequence The data.
	 */
	public function get_data(): TypedSequence {
		$onlinemaps_repository = new OnlineMapsRepository();
		return $onlinemaps_repository->get_all();
	}

	/**
	 * This function handles the table actions.
	 */
	public function handle_table_actions(): void {
		$wp_nonce      = null;
		$nonce_request = filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, '_wpnonce', FILTER_DEFAULT );
		if ( is_string( $nonce_request ) ) {
			$wp_nonce = $nonce_request;
		}

		$action  = filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'action', FILTER_DEFAULT );
		$action2 = filter_input( INPUT_GET, 'action2', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'action2', FILTER_DEFAULT );

		if ( 'bulk-delete' === $action || 'bulk-delete' === $action2 ) {
			$admin_url = admin_url( 'admin.php?page=' . OnlineMaps::SUB_PAGE_NAME );
			if ( ! is_string( $wp_nonce ) ) {
				wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::NoNonce->value, $admin_url ) );
				exit;
			}

			$ids = filter_input( INPUT_GET, 'id', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY ) ?? filter_input( INPUT_POST, 'id', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
			if ( is_array( $ids ) ) {
				$nonce_action = 'bulk-delete';
				if ( isset( $this->_args['plural'] ) && is_string( $this->_args['plural'] ) ) {
					$nonce_action = 'bulk-' . $this->_args['plural'];
				}
				if ( ! wp_verify_nonce( $wp_nonce, $nonce_action ) ) {
					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
					exit;
				}

				$icons_repository = new OnlineMapsRepository();
				try {
					$icons_repository->delete_all_ids( $ids );

					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapsDeleted->value, $admin_url ) );
					exit;
				} catch ( \Exception $exc ) {
					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapsNotDeleted->value, $admin_url ) );
				}
			}
		}
	}

	/**
	 * This function prepares the items.
	 */
	public function prepare_items(): void {
		$this->handle_table_actions();

		$data     = $this->get_data();
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = array(
			'id'   => array( 'id', true ),
			'name' => array( 'name', false ),
		);

		$this->_column_headers = array( $columns, $hidden, $sortable );
		$this->items           = $data->to_array();
	}
}
